#pragma once

#include <iostream>
#include <stdexcept>

namespace rational {

// Represents a rational number, that is, a fraction.
struct Rational
{
    // Constructs the rational representing 0
    Rational();

    // Constructs the rational representing `n`
    Rational(long n);

    // Constructs the rational representing `n/d`; throws
    // std::overflow_error if `d` is 0.
    Rational(long n, long d);

    long num, den;
    // invariants:
    //   den > 0
    //   gcd(num, den) == 1
};

bool operator==(const Rational&, const Rational&);
bool operator!=(const Rational&, const Rational&);
bool operator<(const Rational&, const Rational&);
bool operator<=(const Rational&, const Rational&);
bool operator>(const Rational&, const Rational&);
bool operator>=(const Rational&, const Rational&);

std::ostream& operator<<(std::ostream&, const Rational&);
double to_double(const Rational&);

Rational operator-(const Rational&);
Rational reciprocal(const Rational&);

Rational operator+(const Rational&, const Rational&);
Rational operator*(const Rational&, const Rational&);
Rational operator-(const Rational&, const Rational&);
Rational operator/(const Rational&, const Rational&);

Rational& operator++(Rational&);
Rational& operator--(Rational&);
Rational operator++(Rational&, int);
Rational operator--(Rational&, int);

}
