#include "bintree4.h"

#include <iostream>

using namespace std;

namespace bintree {

struct Tree::Node
{
    string key;
    unsigned value;
    link_t left;
    link_t right;
};

Tree::Tree() : size_(0)
{ }

bool Tree::empty() const {
    return root_ == nullptr;
}

size_t Tree::size() const {
    return size_;
}

bool Tree::member(const string& key) const
{
    return find_subtree(root_, key) != nullptr;
}

unsigned Tree::lookup(const string& key) const
{
    Tree::link_t node = find_subtree(root_, key);
    if (node == nullptr)
        throw Not_found(key);
    else return node->value;
}

void Tree::insert(const string& key, unsigned value)
{
    Tree::link_t& node = find_subtree_mut(root_, key);
    if (node == nullptr) {
        node = make_shared<Tree::Node>();
        node->key = key;
        ++size_;
    }
    node->value = value;
}

Tree::link_t Tree::find_subtree(Tree::link_t link, const string& key)
{
    while (link != nullptr) {
        if (key < link->key) link = link->left;
        else if (key > link->key) link = link->right;
        else return link;
    }

    return nullptr;
}

Tree::link_t& Tree::find_subtree_mut(Tree::link_t& link, const string& key)
{
    if (link == nullptr || link->key == key)
        return link;

    Tree::link_t curr = link;
    for (;;) {
        if (key < curr->key) {
            if (curr->left == nullptr || key == curr->left->key)
                return curr->left;
            else
                curr = curr->left;
        } else {
            if (curr->right == nullptr || key == curr->right->key)
                return curr->right;
            else
                curr = curr->right;
        }
    }
}

void Tree::get_keys_helper(Tree::link_t node, vector<string>& out)
{
    if (node != nullptr) {
        get_keys_helper(node->left, out);
        out.push_back(node->key);
        get_keys_helper(node->right, out);
    }
}

vector<string> Tree::get_keys() const
{
    vector<string> result;
    get_keys_helper(root_, result);
    return result;
}

}
